@@ -69,8 +69,22 @@ struct scan_parameters {
 	size_t extra_ie_size;
 	struct scan_freq_set *freqs;
 	bool flush : 1;
+	bool randomize_mac_addr_hint : 1;
+	const char *ssid;	/* Used for direct probe request */
 };
 
+static inline int scan_bss_addr_cmp(const struct scan_bss *a1,
+					const struct scan_bss *a2)
+{
+        return memcmp(a1->addr, a2->addr, sizeof(a1->addr));
+}
+
+static inline bool scan_bss_addr_eq(const struct scan_bss *a1,
+					const struct scan_bss *a2)
+{
+        return !memcmp(a1->addr, a2->addr, sizeof(a1->addr));
+}
+
 uint32_t scan_passive(uint32_t ifindex, scan_trigger_func_t trigger,
 			scan_notify_func_t notify, void *userdata,
 			scan_destroy_func_t destroy);
@@ -92,8 +106,6 @@ void scan_sched_start(struct l_genl_family *nl80211, uint32_t ifindex,
 			uint32_t scan_interval, scan_func_t callback,
 			void *user_data);
 
-enum security scan_get_security(enum ie_bss_capability bss_cap,
-						const struct ie_rsn_info *info);
 void scan_bss_free(struct scan_bss *bss);
 int scan_bss_rank_compare(const void *a, const void *b, void *user);
 
