@@ -39,7 +39,7 @@
 #include "src/storage.h"
 #include "src/mpdu.h"
 
-#define HWSIM_SERVICE "net.connman.iwd.hwsim"
+#define HWSIM_SERVICE "net.connman.hwsim"
 
 #define HWSIM_RADIO_MANAGER_INTERFACE HWSIM_SERVICE ".RadioManager"
 #define HWSIM_RADIO_INTERFACE HWSIM_SERVICE ".Radio"
@@ -340,6 +340,14 @@ static bool radio_info_match_wiphy_id(const void *a, const void *b)
 	return rec->wiphy_id == id;
 }
 
+static bool radio_info_match_addr0(const void *a, const void *b)
+{
+	const struct radio_info_rec *rec = a;
+	const uint8_t *addr0 = b;
+
+	return !memcmp(rec->addrs[0], addr0, ETH_ALEN);
+}
+
 static bool radio_info_match_addr1(const void *a, const void *b)
 {
 	const struct radio_info_rec *rec = a;
@@ -1171,7 +1179,8 @@ static void hwsim_frame_unref(struct hwsim_frame *frame)
 				frame->flags |= HWSIM_TX_STAT_ACK;
 		}
 
-		send_frame_tx_info(frame);
+		if (frame->src_radio)
+			send_frame_tx_info(frame);
 	}
 
 	l_genl_msg_unref(frame->msg);
@@ -1232,7 +1241,6 @@ static void send_custom_frame_destroy(void *user_data)
 		l_dbus_message_unref(info->user_data);
 
 	l_free(info->frame);
-	l_free(info->radio);
 	l_free(info);
 }
 
@@ -1241,7 +1249,6 @@ static bool send_custom_frame(const uint8_t *addr, uint32_t freq,
 		void *user_data)
 {
 	struct hwsim_frame *frame = l_new(struct hwsim_frame, 1);
-	struct radio_info_rec *radio = l_new(struct radio_info_rec, 1);
 	struct send_frame_info *info = l_new(struct send_frame_info, 1);
 
 	frame->frequency = freq;
@@ -1250,25 +1257,24 @@ static bool send_custom_frame(const uint8_t *addr, uint32_t freq,
 	frame->payload = payload;
 
 	info->frame = frame;
-	info->radio = radio;
 	info->user_data = user_data;
 
-	memcpy(info->radio->addrs[0], addr, ETH_ALEN);
-	memcpy(info->radio->addrs[1], addr, ETH_ALEN);
-
-	/* hwsim expects the first nibble of the address byte to be '4' */
-	info->radio->addrs[1][0] |= 0x40;
+	info->radio = l_queue_find(radio_info, radio_info_match_addr0, addr) ?:
+		l_queue_find(radio_info, radio_info_match_addr1, addr);
+	if (!info->radio)
+		goto error;
 
 	if (!send_frame(info, send_custom_frame_callback,
-			send_custom_frame_destroy)) {
-		l_free(frame);
-		l_free(radio);
-		l_free(info);
-
-		return false;
-	}
+			send_custom_frame_destroy))
+		goto error;
 
 	return true;
+
+error:
+	l_free(frame);
+	l_free(info);
+
+	return false;
 }
 
 struct interface_match_data {
@@ -2117,7 +2123,7 @@ static void request_name_callback(struct l_dbus *dbus, bool success,
 
 static void ready_callback(void *user_data)
 {
-	l_dbus_name_acquire(dbus, "net.connman.iwd.hwsim", false, false, true,
+	l_dbus_name_acquire(dbus, HWSIM_SERVICE, false, false, true,
 				request_name_callback, NULL);
 
 	if (!l_dbus_object_manager_enable(dbus))
@@ -2428,7 +2434,7 @@ int main(int argc, char *argv[])
 	for (;;) {
 		int opt;
 
-		opt = getopt_long(argc, argv, ":L:CD:kn:ipv", main_options,
+		opt = getopt_long(argc, argv, ":L:CD:kn:ipvh", main_options,
 									NULL);
 		if (opt < 0)
 			break;
